# NOTE: this class has been translated from Java.
# Do not modify this file, changes will be overwritten.

#package orbitgt.mapping3d.sdk.viewer;

from .....com.orbitgis.tool.Strings import Strings
from .....orbitgt.mapping3d.sdk.viewer.Coordinate import Coordinate

#
# Class Bounds defines a spatial bounding box in 2D or 3D. It has a coordinate reference system (CRS) and can be valid or invalid.
#

class Bounds:
    # The type of objects in this class */
    TYPE: str = "B"

    # The EPSG CRS code, like "4326" for example */
    #public String crs
    # The minimum X value (if valid) */
    #public double minX
    # The minimum Y value (if valid) */
    #public double minY
    # The minimum Z value (if valid) */
    #public double minZ
    # The maximum X value (if valid) */
    #public double maxX
    # The maximum Y value (if valid) */
    #public double maxY
    # The maximum Z value (if valid) */
    #public double maxZ
    # Do the bounds have a Z value? (if valid) */
    #public boolean is3D
    # Do these bounds have a valid spatial extent? */
    #public boolean valid

    #
    # Create new (invalid) bounds.
    # @param crs the EPSG CRS code of the bounds, like "4326" for example.
    #
    def __init__(self,crs: str):
        self.crs = crs
        self.minX = 0.0
        self.minY = 0.0
        self.minZ = 0.0
        self.maxX = 0.0
        self.maxY = 0.0
        self.maxZ = 0.0
        self.minX = 0.0
        self.minX = 0.0
        self.minX = 0.0
        self.minX = 0.0
        self.is3D = False
        self.valid = False

    #
    # Create new 2D bounds.
    # @param crs the EPSG CRS code of the bounds, like "4326" for example.
    # @param minX the minimum X value.
    # @param minY the minimum Y value.
    # @param maxX the maximum X value.
    # @param maxY the maximum Y value.
    # @return the new bounds.
    #
    def fromXY(crs: str,minX: float,minY: float,maxX: float,maxY: float) -> 'Bounds':
        instance = Bounds(crs)
        instance.addXY(minX,minY)
        instance.addXY(maxX,maxY)
        return instance

    #
    # Create new 3D bounds.
    # @param crs the EPSG CRS code of the bounds, like "4326" for example.
    # @param minX the minimum X value.
    # @param minY the minimum Y value.
    # @param minZ the minimum Z value.
    # @param maxX the maximum X value.
    # @param maxY the maximum Y value.
    # @param maxZ the maximum Z value.
    # @return the new bounds.
    #
    def fromXYZ(crs: str,minX: float,minY: float,minZ: float,maxX: float,maxY: float,maxZ: float) -> 'Bounds':
        instance = Bounds(crs)
        instance.addXYZ(minX,minY,minZ)
        instance.addXYZ(maxX,maxY,maxZ)
        return instance

    #
    # Add a 2D coordinate using X and Y.
    # @param x the X value to add.
    # @param y the Y value to add.
    #
    def addXY(self,x: float,y: float) -> None:
        # Do we have existing bounds ? */
        if (self.valid):
            # Adjust the bounds */
            if (x<self.minX): self.minX = x
            if (x>self.maxX): self.maxX = x
            if (y<self.minY): self.minY = y
            if (y>self.maxY): self.maxY = y
        else:
            # Set the first bounds */
            self.minX = x
            self.minY = y
            self.maxX = x
            self.maxY = y
            self.valid = True

    #
    # Add a 3D coordinate using X, Y and Z.
    # Note that this makes this instance a 3D bounds instance.
    # @param x the X value to add.
    # @param y the Y value to add.
    # @param z the Z value to add.
    #
    def addXYZ(self,x: float,y: float,z: float) -> None:
        # Set 3D */
        self.is3D = True
        # Do we have existing bounds ? */
        if (self.valid):
            # Adjust the bounds */
            if (x<self.minX): self.minX = x
            if (x>self.maxX): self.maxX = x
            if (y<self.minY): self.minY = y
            if (y>self.maxY): self.maxY = y
            if (z<self.minZ): self.minZ = z
            if (z>self.maxZ): self.maxZ = z
        else:
            # Set the first bounds */
            self.minX = x
            self.minY = y
            self.maxX = x
            self.maxY = y
            self.minZ = z
            self.maxZ = z
            self.valid = True

    #
    # Add a coordinate to the bounds.
    # @param coordinate the coordinate to add.
    #
    def addCoordinate(self,coordinate: Coordinate) -> None:
        if (not coordinate.is3D): self.addXY(coordinate.x,coordinate.y)
        if (coordinate.is3D): self.addXYZ(coordinate.x,coordinate.y,coordinate.z)

    #
    # Get the center coordinate.
    # @return returns null if these bounds are not valid.
    #
    def getCenter(self) -> Coordinate:
        if (self.valid==False): return None
        centerX = self.minX+((self.maxX-self.minX)/2.0)
        centerY = self.minY+((self.maxY-self.minY)/2.0)
        centerZ = 0.0
        if (self.is3D): centerZ = self.minZ+((self.maxZ-self.minZ)/2.0)
        return Coordinate(self.crs,centerX,centerY,centerZ,self.is3D)

    #
    # Get the width (along the X axis) of these bounds.
    # @return returns 0.0 if these bounds are not valid.
    #
    def getWidth(self) -> float:
        if (self.valid==False): return 0.0
        return (self.maxX-self.minX)

    #
    # Get the height (along the Y axis) of these bounds.
    # @return returns 0.0 if these bounds are not valid.
    #
    def getHeight(self) -> float:
        if (self.valid==False): return 0.0
        return (self.maxY-self.minY)

    #
    # Get the depth (along the Z axis) of these bounds.
    # @return returns 0.0 if these bounds are not valid or not 3D.
    #
    def getDepth(self) -> float:
        if (self.valid==False): return 0.0
        if (self.is3D==False): return 0.0
        return (self.maxZ-self.minZ)

    #
    # Convert the object to a string.
    # @return the string form of the object.
    #
    def toString(self) -> str:
        if (self.valid==False): return "[Bounds:invalid]"
        if (self.is3D): return "[Bounds:crs="+Strings.toString(self.crs)+",minX="+Strings.toString(self.minX)+",minY="+Strings.toString(self.minY)+",minZ="+Strings.toString(self.minZ)+",maxX="+Strings.toString(self.maxX)+",maxY="+Strings.toString(self.maxY)+",maxZ="+Strings.toString(self.maxZ)+"]"
        return "[Bounds:crs="+Strings.toString(self.crs)+",minX="+Strings.toString(self.minX)+",minY="+Strings.toString(self.minY)+",maxX="+Strings.toString(self.maxX)+",maxY="+Strings.toString(self.maxY)+"]"
